@@ -64,7 +64,6 @@ from pydrake.all import (
     SchunkWsgDriver,
     SchunkWsgPositionController,
     SchunkWsgStatusReceiver,
-    ScopedName,
     SharedPointerSystem,
     SimIiwaDriver,
     SimulatorConfig,
@@ -175,9 +174,7 @@ class Scenario:
         accuracy=1.0e-2,
     )
 
-    plant_config: MultibodyPlantConfig = MultibodyPlantConfig(
-        discrete_contact_approximation="sap"
-    )
+    plant_config: MultibodyPlantConfig = MultibodyPlantConfig()
 
     directives: typing.List[ModelDirective] = dc.field(default_factory=list)
 
@@ -217,9 +214,9 @@ class Directives:
 
 def load_scenario(
     *,
-    filename: str = None,
-    data: str = None,
-    scenario_name: str = None,
+    filename: str | None = None,
+    data: str | None = None,
+    scenario_name: str | None = None,
     defaults: Scenario = Scenario(),
 ):
     warnings.warn("load_scenario is deprecated. Use LoadScenario instead.")
@@ -230,9 +227,9 @@ def load_scenario(
 
 def add_scenario(
     *,
-    filename: str = None,
-    data: str = None,
-    scenario_name: str = None,
+    filename: str | None = None,
+    data: str | None = None,
+    scenario_name: str | None = None,
     defaults: Scenario = Scenario(),
 ):
     warnings.warn("add_directives is deprecated. Use AppendDirectives instead.")
@@ -244,9 +241,9 @@ def add_scenario(
 # TODO(russt): load from url (using packagemap).
 def LoadScenario(
     *,
-    filename: str = None,
-    data: str = None,
-    scenario_name: str = None,
+    filename: str | None = None,
+    data: str | None = None,
+    scenario_name: str | None = None,
     defaults: Scenario = Scenario(),
 ) -> Scenario:
     """Implements the command-line handling logic for scenario data.
@@ -291,9 +288,9 @@ def LoadScenario(
 def AppendDirectives(
     scenario: Scenario,
     *,
-    filename: str = None,
-    data: str = None,
-    scenario_name: str = None,
+    filename: str | None = None,
+    data: str | None = None,
+    scenario_name: str | None = None,
 ) -> Scenario:
     """Append additional directives to an existing scenario.
 
@@ -374,11 +371,11 @@ def _PopulatePlantOrDiagram(
     plant: MultibodyPlant,
     parser: Parser,
     scenario: Scenario,
-    model_instance_names: typing.List[str],
+    model_instance_names: typing.List[str] | None,
     add_frozen_child_instances: bool = True,
     package_xmls: typing.List[str] = [],
-    parser_preload_callback: typing.Callable[[Parser], None] = None,
-    parser_prefinalize_callback: typing.Callable[[Parser], None] = None,
+    parser_preload_callback: typing.Callable[[Parser], None] | None = None,
+    parser_prefinalize_callback: typing.Callable[[Parser], None] | None = None,
 ) -> None:
     """See MakeMultibodyPlant and MakeRobotDiagram for details."""
     if model_instance_names is None:
@@ -401,7 +398,7 @@ def _PopulatePlantOrDiagram(
 
     if not model_instance_names is None:
         tree = DirectivesTree(flattened_directives)
-        directives = tree.GetDirectivesFromRootToModels(model_instance_names)
+        directives = tree.GetDirectivesFromModelsToRoot(model_instance_names)
         children_to_freeze = set()
 
         if add_frozen_child_instances:
@@ -432,11 +429,11 @@ def _PopulatePlantOrDiagram(
 def MakeMultibodyPlant(
     scenario: Scenario,
     *,
-    model_instance_names: typing.List[str] = None,
+    model_instance_names: typing.List[str] | None = None,
     add_frozen_child_instances: bool = False,
     package_xmls: typing.List[str] = [],
-    parser_preload_callback: typing.Callable[[Parser], None] = None,
-    parser_prefinalize_callback: typing.Callable[[Parser], None] = None,
+    parser_preload_callback: typing.Callable[[Parser], None] | None = None,
+    parser_prefinalize_callback: typing.Callable[[Parser], None] | None = None,
 ) -> MultibodyPlant:
     """Use a scenario to create a MultibodyPlant. This is intended, e.g., to facilitate
     easily building subsets of a scenario, for instance, to make a plant for a
@@ -489,11 +486,11 @@ def MakeMultibodyPlant(
 def MakeRobotDiagram(
     scenario: Scenario,
     *,
-    model_instance_names: typing.List[str] = None,
+    model_instance_names: typing.List[str] | None = None,
     add_frozen_child_instances: bool = True,
     package_xmls: typing.List[str] = [],
-    parser_preload_callback: typing.Callable[[Parser], None] = None,
-    parser_prefinalize_callback: typing.Callable[[Parser], None] = None,
+    parser_preload_callback: typing.Callable[[Parser], None] | None = None,
+    parser_prefinalize_callback: typing.Callable[[Parser], None] | None = None,
 ) -> RobotDiagram:
     """Use a scenario to create a RobotDiagram (MultibodyPlant + SceneGraph). This is
     intended, e.g., to facilitate easily building subsets of a scenario, for instance,
@@ -545,8 +542,10 @@ def MakeRobotDiagram(
 class _MultiplexState(LeafSystem):
     def __init__(self, plant: MultibodyPlant, model_instance_names: typing.List[str]):
         LeafSystem.__init__(self)
-        total_states = 0
-        for name in model_instance_names:
+        self._total_states = 0
+        self._model_instance_names = model_instance_names
+        self._plant = plant
+        for name in self._model_instance_names:
             model_instance = plant.GetModelInstanceByName(name)
             num_states = plant.num_multibody_states(model_instance)
             # The logic below assumes num_positions == num_velocities (though
@@ -555,46 +554,53 @@ class _MultiplexState(LeafSystem):
                 model_instance
             )
             self.DeclareVectorInputPort(name + ".state", num_states)
-            total_states += num_states
+            self._total_states += num_states
         self.DeclareVectorOutputPort(
             "combined_state",
-            total_states,
+            self._total_states,
             self.CalcOutput,
         )
 
     def CalcOutput(self, context, output):
         # The order should should be [q, q, ..., v, v, ...].
-        positions = np.array([])
-        velocities = np.array([])
-        for i in range(self.num_input_ports()):
+        positions = np.zeros((self._total_states // 2,))
+        velocities = np.zeros((self._total_states // 2,))
+        for i, model_instance_name in enumerate(self._model_instance_names):
+            model_instance_index = self._plant.GetModelInstanceByName(
+                model_instance_name
+            )
             state = self.get_input_port(i).Eval(context)
             num_q = len(state) // 2
-            positions = np.append(positions, state[:num_q])
-            velocities = np.append(velocities, state[num_q:])
+            self._plant.SetPositionsInArray(
+                model_instance_index, state[:num_q], positions
+            )
+            self._plant.SetVelocitiesInArray(
+                model_instance_index, state[num_q:], velocities
+            )
         output.SetFromVector(np.concatenate((positions, velocities)))
 
 
-class _DemultiplexInput(LeafSystem):
+class _DemultiplexActuation(LeafSystem):
     def __init__(self, plant: MultibodyPlant, model_instance_names: typing.List[str]):
         LeafSystem.__init__(self)
         total_inputs = 0
+        self._plant = plant
         for name in model_instance_names:
-            num_actuators = plant.num_actuators(plant.GetModelInstanceByName(name))
+            model_instance_index = plant.GetModelInstanceByName(name)
+            num_actuators = plant.num_actuators(model_instance_index)
             self.DeclareVectorOutputPort(
                 name + ".input",
                 num_actuators,
-                partial(
-                    self.CalcOutput,
-                    size=num_actuators,
-                    start_index=total_inputs,
-                ),
+                partial(self.CalcOutput, model_instance_index=model_instance_index),
             )
             total_inputs += num_actuators
         self.DeclareVectorInputPort("combined_input", total_inputs)
 
-    def CalcOutput(self, context, output, size, start_index):
+    def CalcOutput(self, context, output, model_instance_index):
         input = self.get_input_port().Eval(context)
-        output.SetFromVector(input[start_index : start_index + size])
+        output.SetFromVector(
+            self._plant.GetActuationFromArray(model_instance_index, input)
+        )
 
 
 # TODO(russt): Use the c++ version pending https://github.com/RobotLocomotion/drake/issues/20055
@@ -684,37 +690,13 @@ def _ApplyDriverConfigSim(
             sim_plant.GetModelInstanceByName(n) for n in model_instance_names
         ]
 
-        controller_plant = MultibodyPlant(time_step=sim_plant.time_step())
-        parser = Parser(controller_plant)
-        for p in package_xmls:
-            parser.package_map().AddPackageXml(p)
-        ConfigureParser(parser)
-
-        # Make the plant for the iiwa controller to use.
-        controller_directives = []
-        for d in FlattenModelDirectives(
-            ModelDirectives(directives=scenario.directives), parser.package_map()
-        ).directives:
-            if d.add_model and (d.add_model.name in model_instance_names):
-                controller_directives.append(d)
-            if (
-                d.add_weld
-                and (
-                    ScopedName.Parse(d.add_weld.child).get_namespace()
-                    in model_instance_names
-                )
-                and (
-                    d.add_weld.parent == "world"
-                    or ScopedName.Parse(d.add_weld.parent).get_namespace()
-                    in model_instance_names
-                )
-            ):
-                controller_directives.append(d)
-        ProcessModelDirectives(
-            directives=ModelDirectives(directives=controller_directives),
-            parser=parser,
+        # Make the controller plant.
+        controller_plant = MakeMultibodyPlant(
+            scenario=scenario,
+            model_instance_names=model_instance_names,
+            add_frozen_child_instances=True,
+            package_xmls=package_xmls,
         )
-        controller_plant.Finalize()
 
         # Add the controller
 
@@ -754,7 +736,7 @@ def _ApplyDriverConfigSim(
                 controller.get_input_port_estimated_state(),
             )
             builder.Connect(
-                controller.get_output_port(),
+                controller.get_output_port_control(),
                 sim_plant.get_actuation_input_port(model_instances[0]),
             )
             builder.ExportOutput(
@@ -763,11 +745,11 @@ def _ApplyDriverConfigSim(
             )
         else:
             combined_state = builder.AddSystem(
-                _MultiplexState(sim_plant, model_instance_names)
+                _MultiplexState(controller_plant, model_instance_names)
             )
             combined_state.set_name(model_instance_name + ".combined_state")
             combined_input = builder.AddSystem(
-                _DemultiplexInput(sim_plant, model_instance_names)
+                _DemultiplexActuation(controller_plant, model_instance_names)
             )
             combined_input.set_name(model_instance_name + ".combined_input")
             for index, model_instance in enumerate(model_instances):
@@ -784,7 +766,7 @@ def _ApplyDriverConfigSim(
                 controller.get_input_port_estimated_state(),
             )
             builder.Connect(
-                controller.get_output_port(), combined_input.get_input_port()
+                controller.get_output_port_control(), combined_input.get_input_port()
             )
             builder.ExportOutput(
                 combined_state.get_output_port(),
@@ -906,14 +888,14 @@ def _ApplyCameraConfigSim(
 
 def MakeHardwareStation(
     scenario: Scenario,
-    meshcat: Meshcat = None,
+    meshcat: Meshcat | None = None,
     *,
     package_xmls: typing.List[str] = [],
     hardware: bool = False,
-    parser_preload_callback: typing.Callable[[Parser], None] = None,
-    parser_prefinalize_callback: typing.Callable[[Parser], None] = None,
-    prebuild_callback: typing.Callable[[DiagramBuilder], None] = None,
-) -> Diagram:
+    parser_preload_callback: typing.Callable[[Parser], None] | None = None,
+    parser_prefinalize_callback: typing.Callable[[Parser], None] | None = None,
+    prebuild_callback: typing.Callable[[DiagramBuilder], None] | None = None,
+) -> RobotDiagram:
     """Make a diagram encapsulating a simulation of (or the communications
     interface to/from) a physical robot, including sensors and controllers.
 
@@ -1381,7 +1363,7 @@ def _ApplyCameraLcmIdInterface(
 
 def _MakeHardwareStationInterface(
     scenario: Scenario,
-    meshcat: Meshcat = None,
+    meshcat: Meshcat | None = None,
     *,
     package_xmls: typing.List[str] = [],
 ) -> Diagram:
@@ -1462,8 +1444,8 @@ def AddPointClouds(
     scenario: Scenario,
     station: Diagram,
     builder: DiagramBuilder,
-    poses_output_port: OutputPort = None,
-    meshcat: Meshcat = None,
+    poses_output_port: OutputPort | None = None,
+    meshcat: Meshcat | None = None,
 ) -> typing.Mapping[str, DepthImageToPointCloud]:
     """
     Adds one DepthImageToPointCloud system to the `builder` for each camera in `scenario`, and connects it to the respective camera station output ports.
