@@ -8,11 +8,18 @@ import numpy as np
 import pydot
 from IPython import get_ipython
 from IPython.display import SVG, display
-from pydrake.all import GetDrakePath
+from pydrake.all import (
+    ApplyLcmBusConfig,
+    ApplyVisualizationConfig,
+    DrakeLcmParams,
+    GetDrakePath,
+    Meshcat,
+    VisualizationConfig,
+)
 from pydrake.common import GetDrakePath
 from pydrake.geometry import RenderLabel
-from pydrake.multibody.parsing import Parser
-from pydrake.systems.framework import System
+from pydrake.multibody.parsing import PackageMap, Parser
+from pydrake.systems.framework import DiagramBuilder, System
 from pydrake.systems.sensors import ImageLabel16I
 
 # Use a global variable here because some calls to IPython will actually cause
@@ -44,11 +51,6 @@ def FindResource(filename: str):
     return os.path.join(os.path.dirname(__file__), filename)
 
 
-def LoadDataResource(filename: str):
-    warn("`LoadDataResource` is deprecated. Use `FindDataResource` instead.")
-    FindDataResource(filename)
-
-
 def FindDataResource(filename: str):
     """
     Returns the absolute path to the given filename relative to the book data
@@ -71,10 +73,31 @@ def FindDataResource(filename: str):
     return path
 
 
+def ConfigurePackageMap(package_map: PackageMap):
+    """Add the `manipulation` module packages to the given PackageMap."""
+    package_xml = os.path.join(os.path.dirname(__file__), "models/package.xml")
+    package_map.AddPackageXml(filename=package_xml)
+
+
 def ConfigureParser(parser: Parser):
     """Add the `manipulation` module packages to the given Parser."""
-    package_xml = os.path.join(os.path.dirname(__file__), "models/package.xml")
-    parser.package_map().AddPackageXml(filename=package_xml)
+    ConfigurePackageMap(parser.package_map())
+
+
+def AddMujocoMenagerie(package_map: PackageMap):
+    assert (
+        False
+    ), "`manipulation.utils.AddMujocoMenagerie` is has moved. Use `manipulation.remotes.AddMujocoMenagerie` instead."
+
+
+def ApplyDefaultVisualization(builder: DiagramBuilder, meshcat: Meshcat):
+    """Follows Drake's ApplyDefaultVisualization, but disables LCM (drake
+    visualizer) by default."""
+    config = VisualizationConfig()
+    lcm_buses = ApplyLcmBusConfig(
+        {"default": DrakeLcmParams(lcm_url="memq://null")}, builder
+    )
+    ApplyVisualizationConfig(config, builder, lcm_buses=lcm_buses, meshcat=meshcat)
 
 
 def colorize_labels(image: ImageLabel16I):
@@ -107,9 +130,25 @@ def colorize_labels(image: ImageLabel16I):
     return color_image
 
 
+def GetDrakeVersion() -> str:
+    """Returns a string representing the Drake version, if availableor "source"."""
+    drake_version_txt = (
+        Path(GetDrakePath()).parent.parent / "doc" / "drake" / "VERSION.TXT"
+    )
+    if drake_version_txt.is_file():
+        with open(drake_version_txt, "r") as f:
+            drake_version = f.read()
+        drake_version = drake_version.split()[0]
+    else:
+        drake_version = "source"
+    return drake_version
+
+
 def DrakeVersionGreaterThan(minimum_date: date):
     """Check that the Drake version is at least `minimum_data`."""
-    drake_version_txt = Path(GetDrakePath()).parent / "doc" / "drake" / "VERSION.TXT"
+    drake_version_txt = (
+        Path(GetDrakePath()).parent.parent / "doc" / "drake" / "VERSION.TXT"
+    )
     version_dates = {
         "1.13.0": date(year=2023, month=2, day=14),
         "1.14.0": date(year=2023, month=3, day=15),
@@ -149,7 +188,7 @@ def DrakeVersionGreaterThan(minimum_date: date):
             )
 
 
-def RenderDiagram(system: System, max_depth: int = None):
+def RenderDiagram(system: System, max_depth: int | None = None):
     """Use pydot to render the GraphViz diagram of the given system.
 
     Args:
